#include "ennemi.h"
#include "mario.h"
#include "box.h"
#include "tile.h"
#include "world.h"
#include "score.h"

#include <stdbool.h>

void display_ennemi(ennemi_t * e)
{
	if (e->life==DEAD)
		return;
	if (e->b.x<=world_get_real_x0(0)-e->b.w || e->b.x>=world_get_real_x0(0)+127)
		return;
	else
		e->discovered=1;

	if (e->type==GOOMBA)
	{
		extern image_t img_goomba;
		tileset_t goomba={&img_goomba, GOOMBA_X, GOOMBA_Y, 1};
		if (e->life==1)
			draw_tile(e->b.x-world_get_real_x0(0), e->b.y-world_get_real_y0(mario.p.y), &goomba, 1+(time_id/10)%2, 0);
		if (e->life==0)
			draw_tile(e->b.x-world_get_real_x0(0), e->b.y-world_get_real_y0(mario.p.y), &goomba, 0, 0);
	}
	if (e->type==KOOPA_VERTE)
	{
		extern image_t img_koopa_verte;
		tileset_t koopa_verte={&img_koopa_verte, KOOPA_X, KOOPA_Y, 1};
		if (e->life==1)
			draw_tile(e->b.x-world_get_real_x0(0), e->b.y-world_get_real_y0(mario.p.y), &koopa_verte, (1+e->p1)+(time_id/8)%2, 0);
	}
	if (e->type==CARAPACE_VERTE)
	{
		extern image_t img_carapace_verte;
		tileset_t carapace_verte={&img_carapace_verte, CARAPACE_X, CARAPACE_Y, 1};
		draw_tile(e->b.x-world_get_real_x0(0), e->b.y-world_get_real_y0(mario.p.y), &carapace_verte, 0, 0);
	}
	
}

bool ennemi_check_collision(ennemi_t *e)
{
	bool x_collide= (mario.p.x<=e->b.x && e->b.x<mario.p.x+mario.p.w-1) || (mario.p.x<=e->b.x+e->b.w-1 && e->b.x+e->b.w<mario.p.x+mario.p.w-1);
	bool y_collide= (mario.p.y<=e->b.y && e->b.y<mario.p.y+mario.p.h-1) || (mario.p.y<=e->b.y+e->b.h-1 && e->b.y+e->b.h<mario.p.y+mario.p.h-1);
	return (x_collide&&y_collide);

}

void hurt_mario()
{
	if (mario.size==M_SMALL && mario_immunity==0)
	{
		mario_dead=1;
		end_level=0;
	}
	else
	{
		mario_smaller();
	}
}

void move_ennemi(ennemi_t *e)
{

	if (e->b.x-e->b.w>world_get_real_x0() && e->b.x<world_get_real_x0()+128)
		e->discovered=1;
	if (e->b.x+e->b.w<=world_get_real_x0())
		e->type=NONE;
	if (e->discovered==0)
		return;
	if (e->life==DEAD)
		e->type=NONE;
	if (e->type==NONE)
		return;
	
	for (int i=0; i<ennemis_global_size; i++)
	{
		ennemi_t* t=&ennemis_global[i];
		if (t!=e && t->discovered && t->type!=NONE)
		{

			bool x_collide= (t->b.x<=e->b.x+e->b.vx && e->b.x+e->b.vx<t->b.x+t->b.w) || (t->b.x<=e->b.x+e->b.w-1+e->b.vx && e->b.x+e->b.w+e->b.vx<t->b.x+t->b.w);
			bool y_collide= (t->b.y<=e->b.y+e->b.vy && e->b.y+e->b.vy<t->b.y+t->b.h) || (t->b.y<=e->b.y+e->b.h-1+e->b.vy && e->b.y+e->b.h+e->b.vy<t->b.y+t->b.h);
			if (x_collide&&y_collide)
			{
				if (e->type!=CARAPACE_VERTE && e->type!=CARAPACE_ROUGE)
				{
					e->b.vx=0;
					e->b.vy=0;
				}
				else
				{
					if (t->type==CARAPACE_VERTE || t->type==CARAPACE_ROUGE)
					{
						e->p1*=-1;
						e->b.vx=3*e->p1;
					}
					else
					{
						e->life=DEAD;
						e->p1=time_id;
						score_add(200);
					}
						
				}
			}
			int sol=0;

			for (int j=0; j<e->b.h; j++)
			{
				gift_t * c=world_get(e->b.x+j ,e->b.y-1);
				if ((c->type==GIFT || c->type==BRICK) && (c->time_hit_id || c->state))
				{
					e->life=DEAD;
					e->p1=time_id;
					score_add(100);
				}
			
			}
		}
	}

	if (e->type==GOOMBA)
	{
		if (e->life==1)
		{
			int old_vx=e->b.vx;
			if (time_id%2)
				e->b.vx=0;

			box_move(&e->b,1);

			if (time_id%2)
				e->b.vx=old_vx;

			if (e->b.vx==0)
			{
				if (e->p1==0)
				{
					e->b.vx=-1;
					e->p1=-1;
				}
				else 
				{
					e->p1*=-1;
					e->b.vx=e->p1;
				}
			}
			if (e->b.y<0)
				e->type=NONE;
			if (ennemi_check_collision(e))
			{
				if (mario.p.vy<=-2)
				{
					e->life=DEAD;
					e->p1=time_id;
					score_add(200);
					mario.p.vy=4;
					mario.p.y=e->b.y+CARAPACE_X;
				}
				else
					hurt_mario();
			}
		}
		else
		{
			if (e->p1>=time_id+10 || e->p1<time_id)
				e->life=DEAD;
		}
	}
	
	if (e->type==KOOPA_VERTE)
	{
		if (e->life==1)
		{
			int old_vx=e->b.vx;
			if (time_id%2)
				e->b.vx=0;

			box_move(&e->b,1);

			if (time_id%2)
				e->b.vx=old_vx;

			if (e->b.vx==0)
			{
				if (e->p1==0)
				{
					e->b.vx=-1;
					e->p1=-1;
				}
				else 
				{
					e->p1*=-1;
					e->b.vx=e->p1;
				}
			}
			if (e->b.y<0)
				e->type=NONE;
			if (ennemi_check_collision(e))
			{
				if (mario.p.vy<=-2)
				{
					e->type=CARAPACE_VERTE;
					e->b.h=CARAPACE_Y;
					e->p1=0;
					e->b.vx=0;
					score_add(200);
					mario.p.vy=4;
					mario.p.y=e->b.y+CARAPACE_Y+1;
				}
				else
					hurt_mario();
			}
		}
		else
				e->life=DEAD;
	}
	if (e->type==CARAPACE_VERTE)
	{
		if (e->life==1)
		{
			box_move(&e->b,0);

			if (e->b.vx==0)
			{
				e->p1*=-1;
				e->b.vx=e->p1;
			}
			if (e->b.y<0)
				e->type=NONE;
			
			if (ennemi_check_collision(e))
			{
				if (e->p1==0)
				{
					if (mario.p.x>=e->b.x)
						e->p1=-1;
					if (mario.p.vy<=e->b.x)
						e->p1=1;
					e->b.vx=3*e->p1;
				}
				else
				{
					if (mario.p.vy<=-2)
					{
						e->p1=0;
						e->b.vx=0;
						mario.p.vy=4;
						mario.p.y=e->b.y+CARAPACE_Y+1;
					}
					else
						hurt_mario();
				}
			}
		}
	}
	display_ennemi(e);
}




ennemi_t * ennemis_global=0;
int ennemis_global_size=0;
void display_ennemi_table()
{
	for (int i=0; i<ennemis_global_size; i++)
	{
		move_ennemi(&ennemis_global[i]);
	}
}